@@ -7,7 +7,7 @@
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
-#   the Free Software Foundation, either version 3 of the License, or
+#   the Free Software Foundation, either version 2 of the License, or
 #   (at your option) any later version.
 #
 #   This program is distributed in the hope that it will be useful,
@@ -81,6 +81,7 @@ rollapply.xts <- function(data, width, FUN, ..., by=1, by.column=TRUE,
     #xx <- sapply(1:NROW(e), function(i) FUN(data[e[i,],],...))
     ##xx <- sapply(ind, function(i) FUN(data[(i-width+1):i,],...))
     xx <- sapply(ind, function(i) FUN(.subset_xts(data,(i-width+1):i),...))
+    if(!is.null(dim(xx))) xx <- t(xx)
     res <- xts(xx, tt, if (by == 1) attr(data, "frequency"))
   } else if( by.column ) {
     res <- xts( sapply( 1:NCOL(data), function(j)
@@ -95,13 +96,14 @@ rollapply.xts <- function(data, width, FUN, ..., by=1, by.column=TRUE,
     if(!is.null(dim(xx))) xx <- t(xx)
     res <- xts(xx, tt, if (by == 1) attr(data, "frequency"))
   }
-  if( na.pad ) {
-    tmp <- merge(res, xts(,idx, attr(data, "frequency")))
-    if(is.null(colnames(res))) {
-      colnames(tmp) <- colnames(res)
-    }
-    res <- tmp
+  
+  ix <- index(data) %in% index(res)
+  tmp <- merge(res, xts(,idx, attr(data, "frequency")))
+  if(is.null(colnames(res))) {
+    colnames(tmp) <- colnames(res)
   }
+  res <- na.fill(tmp, fill, ix)
+
   if( by.column && !is.null(dim(data)) ) {
     colnames(res) <- colnames(data)
   }
