@@ -52,3 +52,35 @@ test.merge_index_contains_Inf <- function() {
   checkException(merge(x, x), silent = TRUE)
 }
 # /end Tests for NA in index
+
+# zero-length fill argument
+test.merge_fill_NULL <- function() {
+  x1 <- .xts(1, 1)
+  x2 <- .xts(2, 2)
+  x <- merge(x1, x2, fill = NULL)
+
+  out <- .xts(matrix(c(1, NA, NA, 2), 2), c(1,2))
+  colnames(out) <- c("x1", "x2")
+  checkIdentical(x, out)
+}
+
+test.merge_fill_zero_length <- function() {
+  x1 <- .xts(1, 1)
+  x2 <- .xts(2, 2)
+  x <- merge(x1, x2, fill = numeric())
+
+  out <- .xts(matrix(c(1, NA, NA, 2), 2), c(1,2))
+  colnames(out) <- c("x1", "x2")
+  checkIdentical(x, out)
+}
+
+test.merge_with_zero_width_returns_original_type <- function() {
+  M1 <- .xts(1:3, 1:3, dimnames = list(NULL, "m1"))
+  for (m in c("double", "integer", "logical", "character")) {
+    m1 <- M1
+    storage.mode(m1) <- m
+    e1 <- .xts(,1:3)
+    m2 <- merge(m1, e1)
+    checkIdentical(m1, m2)
+  }
+}
