@@ -26,15 +26,11 @@
 
 SEXP do_is_ordered (SEXP x, SEXP increasing, SEXP strictly)
 {
-  int i, P=0;
+  int i;
   int nx = LENGTH(x) - 1;
-  SEXP res;
   double *real_x;
   int *int_x;
 
-  PROTECT( res = allocVector(LGLSXP, 1) ); P++;
-  LOGICAL(res)[ 0 ] = 1; /* default to true */
-
   if(TYPEOF(x) == REALSXP) {
   /*
   Check for increasing order, strict or non-strict
@@ -43,36 +39,31 @@ SEXP do_is_ordered (SEXP x, SEXP increasing, SEXP strictly)
   if(LOGICAL(increasing)[ 0 ] == 1) { /* INCREASING */
     if(LOGICAL(strictly)[ 0 ] == 1) { /* STRICTLY INCREASING ( > 0 ) */
       for(i = 0; i < nx; i++) {
-        if( real_x[i+1] - real_x[i] <= 0.0 ) {
-          LOGICAL(res)[ 0 ] = 0;
-          break;
+        if( real_x[i+1] <= real_x[i] ) {
+          return ScalarLogical(0);
         } 
       }
     } else { /* NOT-STRICTLY ( 0 || > 0 ) */
       for(i = 0; i < nx; i++) {
-        if( real_x[i+1] - real_x[i] < 0.0 ) {
-          LOGICAL(res)[ 0 ] = 0;
-          break;
+        if( real_x[i+1] < real_x[i] ) {
+          return ScalarLogical(0);
         } 
       }
     }
- 
   /*
   Check for decreasing order, strict or non-strict
   */
   } else { 
     if(LOGICAL(strictly)[ 0 ] == 1) { /* STRICTLY DECREASING ( < 0 ) */
       for(i = 0; i < nx; i++) {
-        if( real_x[i+1] - real_x[i] >= 0.0 ) {
-          LOGICAL(res)[ 0 ] = 0;
-          break;
+        if( real_x[i+1] >= real_x[i] ) {
+          return ScalarLogical(0);
         } 
       }
     } else { /* NOT-STRICTLY ( 0 || < 0 ) */
       for(i = 0; i < nx; i++) {
-        if( real_x[i+1] - real_x[i] > 0.0 ) {
-          LOGICAL(res)[ 0 ] = 0;
-          break;
+        if( real_x[i+1] > real_x[i] ) {
+          return ScalarLogical(0);
         } 
       }
     }
@@ -85,38 +76,53 @@ SEXP do_is_ordered (SEXP x, SEXP increasing, SEXP strictly)
   */
   int_x = INTEGER(x);
   if(LOGICAL(increasing)[ 0 ] == 1) { /* INCREASING */
+    /* Not increasing order if first element is NA */
+    if( int_x[0] == NA_INTEGER )
+      return ScalarLogical(0);
+
     if(LOGICAL(strictly)[ 0 ] == 1) { /* STRICTLY INCREASING ( > 0 ) */
       for(i = 0; i < nx; i++) {
-        if( int_x[i+1] - int_x[i] <= 0 ) {
-          LOGICAL(res)[ 0 ] = 0;
-          break;
+        if( int_x[i+1] <= int_x[i] ) {
+          if (i == (nx-1) && int_x[i+1] == NA_INTEGER) {
+            continue; /* OK if NA is last element */
+          }
+          return ScalarLogical(0);
         } 
       }
     } else { /* NOT-STRICTLY ( 0 || > 0 ) */
       for(i = 0; i < nx; i++) {
-        if( int_x[i+1] - int_x[i] < 0 ) {
-          LOGICAL(res)[ 0 ] = 0;
-          break;
+        if( int_x[i+1] < int_x[i] ) {
+          if (i == (nx-1) && int_x[i+1] == NA_INTEGER) {
+            continue; /* OK if NA is last element */
+          }
+          return ScalarLogical(0);
         } 
       }
     }
- 
   /*
   Check for decreasing order, strict or non-strict
   */
   } else { /* DECREASING */
+    /* Not decreasing order if last element is NA */
+    if( int_x[nx] == NA_INTEGER )
+      return ScalarLogical(0);
+
     if(LOGICAL(strictly)[ 0 ] == 1) { /* STRICTLY DECREASING ( < 0 ) */
       for(i = 0; i < nx; i++) {
-        if( int_x[i+1] - int_x[i] >= 0 ) {
-          LOGICAL(res)[ 0 ] = 0;
-          break;
+        if( int_x[i+1] >= int_x[i] ) {
+          if (i == 0 && int_x[i] == NA_INTEGER) {
+            continue; /* OK if NA is first element */
+          }
+          return ScalarLogical(0);
         } 
       }
     } else { /* NOT-STRICTLY ( 0 || < 0 ) */
       for(i = 0; i < nx; i++) {
-        if( int_x[i+1] - int_x[i] > 0 ) {
-          LOGICAL(res)[ 0 ] = 0;
-          break;
+        if( int_x[i+1] > int_x[i] ) {
+          if (i == 0 && int_x[i] == NA_INTEGER) {
+            continue; /* OK if NA is first element */
+          }
+          return ScalarLogical(0);
         } 
       }
     }
@@ -125,6 +131,6 @@ SEXP do_is_ordered (SEXP x, SEXP increasing, SEXP strictly)
   } else {
     error("'x' must be of type double or integer");
   }
-  UNPROTECT(P);
-  return res;
+
+  return ScalarLogical(1);  /* default to true */
 }
